from pageo.base_page import BasePage
from pageo.locators import IdLocator
from pageo.locators import ClassNameLocator
from pageo.locators import CSSLocator
from pageo.locators import NameLocator
from pageo.locators import LinkTextLocator
from pageo.locators import PartialLinkTextLocator
from pageo.locators import XPATHLocator
from pageo.locators import TagNameLocator



from pageo.locators.id_locator import IdLocator
from pageo.locators.class_name_locator import ClassNameLocator
from pageo.locators.css_locator import CSSLocator
from pageo.locators.name_locator import NameLocator
from pageo.locators.link_text_locator import LinkTextLocator
from pageo.locators.partial_link_text_locator import PartialLinkTextLocator
from pageo.locators.xpath_locator import XPATHLocator
from pageo.locators.tag_name_locator import TagNameLocator



from collections import UserDict
from typing import Hashable, Callable, Any, Dict


class MapDict(UserDict):
    def __init__(self, another_dict: Dict[Hashable, Any], function: Callable[[Any], Any]):
        self.function = function
        super().__init__(another_dict)

    def __getitem__(self, key: Hashable):
        value = self.data.__getitem__(key)
        changed_value = self.function(value)
        return changed_value



from abc import ABC


class AbstractLocator(ABC):
    """
    Вспомогательный абстрактный класс для BasePage, инкапсулирующий логику поиска
    элементов по локаторам.

    Класс работает через протокол дескрипторов. Экземпляр отнаследованного класса представляет один
    локатор, скрывая логику поиска элементов.

    Пример использования наследника в связке с BasePage:
    >>> from pageo.base_page import BasePage
    >>> from pageo.locators.id_locator import IdLocator
    >>>
    >>>
    >>> class AboutPage(BasePage):
    ...     search_field_element = IdLocator("id-search-field")
    ...     def is_search_field(self):
    ...         return True if self.search_field_element else False
    """
    def __init__(self, selector: str, is_many: bool = False, timeout: int = 5):
        self.selector = selector
        self.is_many = is_many
        self.timeout = timeout

    def __get__(self, instance, owner=None):
        if self.is_many:
            return instance._find_elements(self.by, self.selector, self.timeout)
        return instance._find_element(self.by, self.selector, self.timeout)

    def set_name(self, name):
        self.name = name



from threading import Lock

from pageo.errors import DoublePageDefenitionError


class MetaBasePage(type):
    """
    Мета-класс для базового класса BasePage. Создает в классах страницы словарь, где:
    * Ключ - имя класса;
    * Значение - список унаследованных классов от класса, указанного в ключе.
    
    Также проверяет, что нет двух классов страниц с одинаковым именем.
    """
    lock = Lock()

    def __new__(mcls, name, bases, attrs):
        class_instance = super(MetaBasePage, mcls).__new__(mcls, name, bases, attrs)
        if len(class_instance.__mro__) > 2:
            parent_class = class_instance.__mro__[1]
            with mcls.lock:
                if 'inherited_classes' not in parent_class.__dict__:
                    parent_class.inherited_classes = {}
                if parent_class.inherited_classes.get(name) is not None:
                    raise DoublePageDefenitionError("You can't to define two classes with the same name.")
                parent_class.inherited_classes[name] = class_instance
        return class_instance



from selenium.webdriver.common.by import By

from pageo.locators.abstract_locator import AbstractLocator


class LinkTextLocator(AbstractLocator):
    """
    Класс локатора, выполняющий поиск на основе текста внутри ссылки элемента.
    """
    by = By.LINK_TEXT



from tests.integrations import TestPage


def test_caption_with_locator(driver, local_server_url):
    """
    Тест создает экземпляр страницы TestPage и проверяет, что текст в элементе
    с классом 'caption' на тестовой странице соответствует ожидаемому.
    """
    page = TestPage(driver=driver, base_url=local_server_url)
    actual_caption = page.get_caption_text()
    expected_caption = 'caption'
    assert actual_caption == expected_caption


def test_paragraphs_with_locator(driver, local_server_url):
    """
    Тест создает экземпляр страницы TestPage и проверяет, что тексты в элементах
    с классом 'paragraphs' на тестовой странице соответствует ожидаемому.
    """
    page = TestPage(driver=driver, base_url=local_server_url)
    actual_paragraph_texts = page.get_paragraph_texts()
    expected_paragraph_texts = ['first_paragraph', 'second_paragraph']
    assert actual_paragraph_texts == expected_paragraph_texts


def test_caption_with_find_elements(driver, local_server_url):
    """
    Тест создает экземпляр страницы TestPage и проверяет, что текст в элементе
    с классом 'caption' на тестовой странице соответствует ожидаемому.
    """
    page = TestPage(driver=driver, base_url=local_server_url)
    actual_caption_list = page.get_caption_text_list()
    expected_caption = 'caption'
    assert isinstance(actual_caption_list, list)

    actual_caption = actual_caption_list[0].text
    assert actual_caption == expected_caption


def test_paragraphs_with_find_elements(driver, local_server_url):
    """
    Тест создает экземпляр страницы TestPage и проверяет, что тексты в элементах
    с классом 'paragraphs' на тестовой странице соответствует ожидаемому.
    """
    page = TestPage(driver=driver, base_url=local_server_url)
    actual_paragraph_texts = page.get_paragraph_texts_list()
    expected_paragraph_texts = ['first_paragraph', 'second_paragraph']
    assert actual_paragraph_texts == expected_paragraph_texts




from selenium.webdriver.common.by import By

from pageo.locators.abstract_locator import AbstractLocator


class CSSLocator(AbstractLocator):
    """
    Класс локатора, выполняющий поиск на основе css-локатора элемента.
    """
    by = By.CSS_SELECTOR



from setuptools import setup, find_packages


with open("README.md", "r", encoding='utf-8') as readme_file:
    readme = readme_file.read()

__version__ = '0.3.1'

setup(
    name='pageo',
    version=__version__,
    packages=find_packages(exclude=['tests']),
    description='Библиотека на базе Selenium с расширенным функционалом. Предназначена для работы с любым веб-сайтом и его страницами',
    long_description=readme,
    long_description_content_type='text/markdown',
    author='dmitry',
    author_email='dmitryvshivtsev@gmail.com',
    install_requires=[
        'pytest>=7.4.0',
        'selenium>=4.10.0',
    ],
    classifiers=[
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'Operating System :: MacOS :: MacOS X',
        'Operating System :: Microsoft :: Windows',
        'Operating System :: POSIX',
    ],
)

